#!/usr/bin/perl -w
# Mark Kamichoff <prox@prolixium.com>
# Version 0.1
# Checks the PTR and A DNS RRs for the given network
# Takes an optional '-n' to disable color, and a mandatory prefix/len

# Copyright (c) 2007, Mark Kamichoff
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Mark Kamichoff ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Mark Kamichoff BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

use Socket;
use Net::CIDR qw(cidr2octets);
use Term::ANSIColor;

# check argument(s)
if(@ARGV == 1) {
	$color=1;
	$prefixlen=$ARGV[0];
} elsif(@ARGV == 2) {
	if($ARGV[0] eq '-n') {
		$color=0;
		$prefixlen=$ARGV[1];
	} else {
		print "Error: Incorrect arguments.\n";
		syntax();
	}
} else {
	print "Error: Incorrect number of arguments.\n";
	syntax();
}

# check if they're ok
if($prefixlen !~ m/^(\d{1,3}\.){3}\d{1,3}\/\d{1,2}$/) {
	print "Error: Invalid prefix/len.\n";
	syntax();
}

@octet_list = cidr2octets($prefixlen);

foreach $var (@octet_list) {
	if($var =~ m/(\d+)\.(\d+)\.(\d+)\.(\d+)/) {
		lookupip("$1.$2.$3.$4");
	} elsif($var =~ m/(\d+)\.(\d+)\.(\d+)/) {
		for($i=0;$i<255;$i++) {
			lookupip("$1.$2.$3.$i");
		}
	} elsif($var =~ m/(\d+)\.(\d+)/) {
		for($j=0;$j<255;$j++) {
			for($i=0;$i<255;$i++) {
				lookupip("$1.$2.$j.$i");
			}
		}
	} elsif($var =~ m/(\d+)/) {
		for($k=0;$k<255;$k++) {
			for($j=0;$j<255;$j++) {
				for($i=0;$i<255;$i++) {
					lookupip("$1.$k.$j.$i");
				}
			}
		}
	} else {
		die "Sorry, I'd rather not scan all possible IPv4 addresses.\n";
	}
}

sub lookupip {
	$string = pop @_;
	print "PTR $string -> ";
	if($dnsname = gethostbyaddr(inet_aton($string), AF_INET)) {
		print $dnsname;
		print ", A \$_ -> ";

		# check for A's
		if(($_name,$_aliases,$_addrtype,$_length,@addrs) = gethostbyname($dnsname)) {
			$found=$n=0;

			# check for the IP of the PTR in the A's
			while($n<(scalar @addrs)) {
				@Quads = unpack('C4', $addrs[$n]);
				$ipstring = join('.', @Quads);

				# if found, update $found and maybe start green color
				if($ipstring eq $string) {
					$found=1;
					if($color) { print color 'green'; }
					else { print '*'; }
				} elsif($color) { print color 'red'; }	# not found, maybe start red color
				print $ipstring;	# print the IP
				if($color) { print color 'reset'; }	# reset color
				$n += 1;
				if($n<(scalar @addrs)) { print ", "; }
			}
			
			# if we got A from PTR, did we find the IP in the A's?
			if($found) {
				print " [OK]";
			} else {
				print " [MISMATCH]";
			}

		} else {
			print "NXDOMAIN";
		}
	} else {
		print "NXDOMAIN";
	}
	print "\n";
}

sub syntax {
	print "Syntax: $ENV{'_'} [-n] [prefix/len]\n";
	print "Example: $ENV{'_'} 10.10.220.0/24\n";
	exit;
}
